#!/usr/bin/ruby
#
# Buyer beware: This can take some time (e.g. 10-30 mins) to run on a
# decently sized data set.  Output from a run on the default dataset
# at revision 104, 4/15/2006, is given below.
# 
# Starting at Sat Apr 15 23:44:06 PDT 2006
# Reading data from bin/../data/certain_nonzero_data_points, size  11517  46068 325988 bin/../data/certain_nonzero_data_points
# Optimally biased coin, log10(prob)					= -2780.61787927167
# Global equal epsilon, global threshold, log10(prob)			= -2364.20342306969
# Global different epsilon, global threshold, log10(prob)			= -2239.21037794004
# Per-question equal epsilon, per-question threshold, log10(prob)		= -1997.1041138485
# Per-question different epsilon, per-question threshold, log10(prob)	= -1873.11129131795
# Per-nugget equal epsilon, per-nugget threshold, log10(prob)		= -1590.04090774305
# Per-nugget different epsilon, per-nugget threshold, log10(prob)		= -1595.07989902791
# Per-data-point different epsilon, per-data-point threshold, log10(prob)	= -3466.96246006269
# Global different epsilon, per-question threshold, log10(prob)		= -1977.52590331999
# Global different epsilon, per-nugget threshold, log10(prob)		= -1546.53572509758
# Done at Sun Apr 16 00:21:37 PDT 2006
#
# This run was repeated with the same results but in four minutes
# (albeit on a considerably faster machine) on 10/8/2006, revision 204.

require File.dirname(__FILE__) + "/../data_set.rb"
require File.dirname(__FILE__) + "/../environment.rb"

Environment.execute do |options|
  filename = ARGV[0] || File.dirname(__FILE__) + "/../data/certain_nonzero_data_points"
  STDERR.puts "Reading data from #{filename}, size #{`wc #{filename}`}" unless options[:quiet]

  big_data_group = DataGroup.parse_from_file(filename)
  puts "Optimally biased coin, log10(prob)\t\t\t\t\t= #{big_data_group.log10_prob_biased_coin}"
  puts "Global equal epsilon, global threshold, log10(prob)\t\t\t= #{big_data_group.log10_prob_marginal_equal_epsilon_marginal_threshold}"
  puts "Global different epsilon, global threshold, log10(prob)\t\t\t= #{big_data_group.log10_prob_marginal_different_epsilon_marginal_threshold}"

  qdataset = DataSet.parse_from_file(filename, :question)
  puts "Per-question equal epsilon, per-question threshold, log10(prob)\t\t= #{qdataset.log10_prob_separate_equal_epsilon_and_threshold}"
  puts "Per-question different epsilon, per-question threshold, log10(prob)\t= #{qdataset.log10_prob_separate_different_epsilon_and_threshold}"

  ndataset = DataSet.parse_from_file(filename, :nugget)
  puts "Per-nugget equal epsilon, per-nugget threshold, log10(prob)\t\t= #{ndataset.log10_prob_separate_equal_epsilon_and_threshold}"
  puts "Per-nugget different epsilon, per-nugget threshold, log10(prob)\t\t= #{ndataset.log10_prob_separate_different_epsilon_and_threshold}"

  dataset = DataSet.parse_from_file(filename) { |pt| pt }
  puts "Per-data-point different epsilon, per-data-point threshold, log10(prob)\t= #{dataset.log10_prob_separate_different_epsilon_and_threshold}"

  qdataset.verbosity = :quiet
  ndataset.verbosity = :quiet
  puts "Global different epsilon, per-question threshold, log10(prob)\t\t= #{qdataset.log10_prob_global_different_epsilon_local_threshold}"
  puts "Global different epsilon, per-nugget threshold, log10(prob)\t\t= #{ndataset.log10_prob_global_different_epsilon_local_threshold}"
end
