require 'test/unit'
require File.dirname(__FILE__) + "/executable_test_helper.rb"

class ExecutionTest < Test::Unit::TestCase

  include ExecutableTestHelper

  def test_global_map_threshold
    output = run_executable("global_map_threshold.rb")
    assert_global_threshold_present output
    assert_no_qid_thresholds output
    assert_no_nid_thresholds output
    assert_in_delta 0.550169874645249, extract_global_threshold(output), 1E-15
  end

  def test_qid_map_thresholds
    output = run_executable("qid_map_thresholds.rb")
    assert_no_global_thresholds output
    assert_qid_threshold_present output
    assert_equal 5, count_qid_thresholds(output)
    assert_no_nid_thresholds output
  end

  def test_nid_map_thresholds
    output = run_executable("nid_map_thresholds.rb")
    assert_no_global_thresholds output
    assert_no_qid_thresholds output
    assert_nid_threshold_present output
    assert_equal 6, count_nid_thresholds(output)
  end

  def test_simple_qid_nid_selection_thresholds
    output = run_executable("simple_qid_nid_selection_thresholds.rb")
    assert_no_global_thresholds output
    assert_qid_threshold_present output
  end

  def test_simple_gl_qid_selection_thresholds
    output = run_executable("simple_gl_qid_selection_thresholds.rb")
    assert_global_threshold_present output
    assert_qid_threshold_present output
    assert_no_nid_thresholds output
    assert_questions_considered output, [2222, 2112, 2366, "qid42", 2385]
    assert_in_delta 0.550169874645249, extract_global_threshold(output), 1E-15
  end

  def test_simple_gl_nid_selection_thresholds
    output = run_executable("simple_gl_nid_selection_thresholds.rb")
    assert_global_threshold_present output
    assert_no_qid_thresholds output
    assert_nid_threshold_present output
    assert_nuggets_considered output, [[2222, 1], [2222, 3], [2112, 6], [2366, 2], ["qid42", 5], [2385, 4]]
    assert_in_delta 0.550169874645249, extract_global_threshold(output), 1E-15
  end

  def test_simple_gl_qid_nid_selection_thresholds
    output = run_executable("simple_gl_qid_nid_selection_thresholds.rb")
    assert_global_threshold_present output
    assert_qid_threshold_present output
    assert_questions_considered output, [2222, 2112, 2366, "qid42", 2385]
    assert_in_delta 0.550169875, extract_global_threshold(output), 1E-7
  end

  def test_compare_models
    output = run_executable("compare_models.rb #{NEW_FORMAT_DATA_FILE}")
  end
  
end
