#!/usr/bin/ruby

#require 'profile'
require File.dirname(__FILE__) + "/../data_set.rb"
require File.dirname(__FILE__) + "/../parsing/nugget_data.rb"
require File.dirname(__FILE__) + "/../parsing/qresult_request.rb"
require File.dirname(__FILE__) + "/../environment.rb"

# Assumes that epsilons are global and thresholds are per-nugget.
# Collects training data from the given file argument.  Collects
# nugget vitality information from the file specified with
# --answer_key.  Listens for judgement requests on stdin and answers
# them as they come in.  Also listens for qresult requests and answers
# them as they come in, using all relevant preceding judgements.

Environment.execute do |options|

  training = ARGV[0]
  dataset = DataSet.parse_from_file(training, :nugget)
  dataset.verbosity = :quiet if options[:quiet]
  nuggets = NuggetData.parse_from_files(options[:answer_keys])
  accumulated_judgement_requests = []

  STDIN.each do |line|

    if JudgementRequest.line_matches?(line)
      request = JudgementRequest.parse_line(line)
      accumulated_judgement_requests.push request
      nugget_probs = dataset.nugget_probabilities(request)
      # Hash.inspect?
      puts "NUGGET_PROBS_LIST\t" + nugget_probs.map { |key, val| "#{key}=>#{val}" }.join(", ")
      STDOUT.flush
    elsif QResultRequest.line_matches?(line)
      qid = QResultRequest.parse_line(line).qid
      relevant_requests = accumulated_judgement_requests.select do |req|
        req.qid == qid
      end
      f_measure = dataset.expected_f(relevant_requests,
                                     nuggets.vitalities[qid])
      ans = [f_measure.vital_count,
             nuggets.num_vital(qid),
             f_measure.okay_count,
             nuggets.num_okay(qid),
             f_measure.vital_count + f_measure.okay_count,
             nuggets.num_vital(qid) + nuggets.num_okay(qid),
             f_measure.length,
             f_measure.allowance,
             f_measure.recall,
             f_measure.precision,
             f_measure.f_measure]
      puts "QRESULT_RESPONSE\t" + ans.join("\t")
      STDOUT.flush
    end

  end
end
