#!/usr/bin/ruby

#require 'profile'
require File.dirname(__FILE__) + "/../data_set.rb"
require File.dirname(__FILE__) + "/../environment.rb"

Environment.execute do |options|

  dataset = DataSet.parse_from_stream(STDIN, :nugget)
  dataset.verbosity = :quiet if options[:quiet]
  best_epsilon = dataset.best_epsilon

  gldataset = dataset.reallocate(:one_group)
  gldataset.verbosity = :quiet if options[:quiet]
  glthresh = gldataset.only_group.best_threshold(best_epsilon[0], best_epsilon[1])
  puts "GLOBAL\t#{sprintf options[:number_format], glthresh.cutoff}"

  qdataset = dataset.reallocate(:question)
  qdataset.verbosity = :quiet if options[:quiet]
  qdataset.data_groups.each do |group|
    if group.log10_prob_fixed_epsilon_marginal_threshold(best_epsilon[0], best_epsilon[1]) >
        group.log10_prob_fixed_threshold_fixed_epsilon(glthresh, best_epsilon[0], best_epsilon[1])
      qthresh = group.best_threshold(best_epsilon[0], best_epsilon[1])
      puts "QUESTION\t#{group.question_id}\t#{sprintf options[:number_format], qthresh.cutoff}"
      subset = DataSet.allocate(group.data_points, :nugget)
      subset.data_groups.each do |subgroup|
        if subgroup.log10_prob_fixed_epsilon_marginal_threshold(best_epsilon[0], best_epsilon[1]) > 
            subgroup.log10_prob_fixed_threshold_fixed_epsilon(qthresh, best_epsilon[0], best_epsilon[1])
          puts "nugget-level won at question #{subgroup.question_id}, nugget #{subgroup.nugget_id}." unless options[:quiet]
          puts "NUGGET\t#{subgroup.question_id}\t#{subgroup.nugget_id}\t#{sprintf options[:number_format], subgroup.best_threshold(best_epsilon[0], best_epsilon[1]).cutoff}"
        else
          puts "question-level won at question #{subgroup.question_id}, nugget #{subgroup.nugget_id}." unless options[:quiet]
        end
      end
    else
      puts "global won at question #{group.question_id}." unless options[:quiet]
    end
  end

end
