require File.dirname(__FILE__) + "/general_utilities/general_utilities.rb"

# Maintains counters of how many data points there are that
# are judged assigned or not, correctly or incorrectly.

class Counters

  attr_reader :no_right, :no_wrong, :yes_right, :yes_wrong
  
  def initialize(no_right, no_wrong, yes_right, yes_wrong)
    @no_right = no_right
    @no_wrong = no_wrong
    @yes_right = yes_right
    @yes_wrong = yes_wrong
  end

  def step_over!(judgement)
    case judgement
    when :yes
      @yes_right -= 1
      @no_wrong += 1
    when :no
      @yes_wrong -= 1
      @no_right += 1
    when :maybe
    else
      raise "Unknown judgement!"
    end
  end

  def step_over(judgement)
    case judgement
    when :yes:   Counters.new(@no_right, @no_wrong + 1, @yes_right - 1, @yes_wrong)
    when :no:    Counters.new(@no_right + 1, @no_wrong, @yes_right, @yes_wrong - 1)
    when :maybe: Counters.new(@no_right, @no_wrong, @yes_right, @yes_wrong)
    else
      raise "Unknown judgement!"
    end
  end

  def total_right
    @no_right + @yes_right
  end

  # The ratio between the number of new mistaken 'no' judgements
  # introduced by the given counter relative this one to the total
  # number of new 'no' judgements introduced by the given counter.  If
  # the given counter is a counter from the same group, the ratio will
  # be nonnegative.  Returns 0 if no new judgements are introduced.
  def new_mistake_ratio(next_counters)
    new_mistakes = next_counters.no_wrong - @no_wrong
    new_correct_judgements = next_counters.no_right - @no_right
    return 0 if (new_mistakes + new_correct_judgements) == 0
    new_mistakes * 1.0 / (new_mistakes + new_correct_judgements)
  end

  def probability(eps_low, eps_high)
    eps_low**@no_wrong * (1 - eps_low)**@no_right *
      eps_high**@yes_wrong * (1 - eps_high)**@yes_right
  end

  def probability_log10(eps_low, eps_high)
    Math.log10(eps_low) * @no_wrong + Math.log10(1 - eps_low) * @no_right +
      Math.log10(eps_high) * @yes_wrong + Math.log10(1 - eps_high) * @yes_right
  end

  def log10_integral_equal_epsilon_to_one
    Polynomial.integral_x_one_minus_x_powers_from_zero_to_one_log10(@no_wrong + @yes_wrong, @no_right + @yes_right)
  end

  def log10_integral_different_epsilon_to_one
    Polynomial.integral_x_one_minus_x_powers_from_zero_to_one_log10(@no_wrong, @no_right) + 
      Polynomial.integral_x_one_minus_x_powers_from_zero_to_one_log10(@yes_wrong, @yes_right)
  end

  def to_s
    "#{@no_right} #{@no_wrong} #{@yes_right} #{yes_wrong}"
  end

  def ==(other)
    @no_right == other.no_right &&
      @no_wrong == other.no_wrong &&
      @yes_right == other.yes_right &&
      @yes_wrong == other.yes_wrong
  end

end
