require 'optparse'
require File.dirname(__FILE__) + "/general_utilities/general_utilities.rb"

class Environment

  def self.execute(version = "0.1")
    options = parse!(ARGV, version)
    STDERR.puts "Starting at #{`date`}" unless options[:quiet]

    yield options

    if SelectiveProfiler.global.stats_collected?
      STDERR.puts SelectiveProfiler.global.report unless options[:quiet]
    end
    STDOUT.flush
    STDERR.puts "Done at #{`date`}" unless options[:quiet]
  end

  def self.parse!(args, version)
    options = {:number_format => "%1.9f"}

    opts = OptionParser.new do |opts|
      opts.on("-q", "--quiet", "Report only serious errors") do |q|
        options[:quiet] = q
      end

      opts.on("--answer_key FILENAME", "Specify FILENAME containing the official answer key") do |filename|
        options[:answer_keys] ||= []
        options[:answer_keys] = options[:answer_keys] + [ filename ]
      end

      opts.on("--answer_keys FILE1,FILE2,...", Array, "Specify FILENAME containing the official answer key") do |filenames|
        options[:answer_keys] ||= []
        options[:answer_keys] = options[:answer_keys] + filenames
      end

      opts.on("--number_format FORMAT", "Specify the (printf-like) format string wherewith fractions should be reported") do |format|
        options[:number_format] = format
      end

      opts.on("--runtag RUNTAG", "Specify the tag of the run whose answers are being processed") do |tag|
        options[:runtag] = tag
      end

      opts.on_tail("--version", "Show version string and exit") do
        puts version
        exit
      end
    end

    opts.parse!(args)
    options
  end

end
