require 'test/unit'
require File.dirname(__FILE__) + "/error_rate_space.rb"
require File.dirname(__FILE__) + "/data_group.rb"
require File.dirname(__FILE__) + "/test_helper.rb"

class ErrorRateSpaceTest < Test::Unit::TestCase

  include TestHelper

  def setup
    @one_data_point = DataGroup.new([ DataPoint.new(0, 0, 0.5, :yes) ])
    @artificial_data = DataGroup.parse_from_file(ARTIFICIAL_POINTS)
    @data = DataGroup.parse_from_file(SMALL_DATA_FILE)
    @more_data = DataGroup.parse_from_file(NEW_FORMAT_DATA_FILE)
    @more_artificial_data = DataGroup.parse_from_file(LEVEL_TEST_DATA_FILE)
  end

  def error_rate_list(data_group)
    ErrorRateSpace.new(data_group).list
  end

  def mistake_ratios(data_group)
    error_rate_list(data_group).map do |boundary|
      boundary.mistake_ratio
    end
  end
    
  def cutoffs(data_group)
    error_rate_list(data_group).map do |boundary|
      boundary.threshold.cutoff
    end
  end

  def test_construction
    assert_equal([ErrorClassBoundary.new(0, :threshold => @one_data_point.intervaled_thresholds[0]),
                  ErrorClassBoundary.new(1, :threshold => @one_data_point.intervaled_thresholds[1])],
                 error_rate_list(@one_data_point))
    assert_equal([ErrorClassBoundary.new(0, :threshold => @artificial_data.intervaled_thresholds[2]),
                  ErrorClassBoundary.new(1, :threshold => @artificial_data.intervaled_thresholds[3])],
                 error_rate_list(@artificial_data))

    assert_close_arrays([0, 0.085714285714285, 1.0],
                        mistake_ratios(@more_data))
    assert_close_arrays([0.01740771870, 0.09169690414, 0.5501698746],
                        cutoffs(@more_data))
    assert_close_arrays([0, 2/3.0, 6/7.0, 1],
                        mistake_ratios(@more_artificial_data))
    assert_close_arrays([0.15, 0.35, 0.55, 0.8],
                        cutoffs(@more_artificial_data))
  end

  def best_cutoff(data_group, eps_low, eps_high)
    ErrorRateSpace.new(data_group).max_likelihood_threshold(eps_low, eps_high).cutoff
  end

  def test_selection
    # According to Mathematica, the inequality
    # x^2(1-x) > y(1-y)^2, y = 0.1, 0 < x < 1 - y
    # is true when 0.354138 < x < 0.9, and the inequality
    # x^6(1-x) > y(1-y)^6, y = 0.1, 0 < x < 1 - y
    # becomes true when 0.805642 < x < 0.9.
    assert_in_delta 0.15, best_cutoff(@more_artificial_data, 0.1, 0.1), 1E-7
    assert_in_delta 0.15, best_cutoff(@more_artificial_data, 0.35, 0.1), 1E-7
    assert_in_delta 0.35, best_cutoff(@more_artificial_data, 0.36, 0.1), 1E-7
    assert_in_delta 0.35, best_cutoff(@more_artificial_data, 0.5, 0.1), 1E-7
    assert_in_delta 0.35, best_cutoff(@more_artificial_data, 0.8, 0.1), 1E-7
    assert_in_delta 0.55, best_cutoff(@more_artificial_data, 0.81, 0.1), 1E-7

    # For y = 0.3, the first inequality is true for 0.632183 < x < 0.7,
    # and the second is not true for any x < 0.7
    assert_in_delta 0.15, best_cutoff(@more_artificial_data, 0.3, 0.3), 1E-7
    assert_in_delta 0.15, best_cutoff(@more_artificial_data, 0.63, 0.3), 1E-7
    assert_in_delta 0.35, best_cutoff(@more_artificial_data, 0.64, 0.3), 1E-7
    assert_in_delta 0.35, best_cutoff(@more_artificial_data, 0.699, 0.3), 1E-7
  end

end
