require 'test/unit'
require File.dirname(__FILE__) + "/array.rb"

class ArrayTest < Test::Unit::TestCase

  def test_binary_search
    ary = [10, 20, 30, 45, 50, 73]
    assert_equal nil, ary.binary_search(10)
    assert_equal 10, ary.binary_search(11)
    assert_equal 30, ary.binary_search(35)
    assert_equal 45, ary.binary_search(50)
    assert_equal 50, ary.binary_search(51)
    assert_equal 73, ary.binary_search(80)
    assert_equal nil, ary.binary_search(-3)
    assert_equal nil, [].binary_search(-3)
  end

  def test_custom_comparison_function
    ary = [73, 50, 45, 30, 20, 10]
    assert_equal 20, ary.binary_search(10) { |key, obj| obj <=> key }
    assert_equal 20, ary.binary_search(11) { |key, obj| obj <=> key }
    assert_equal 45, ary.binary_search(35) { |key, obj| obj <=> key }
    assert_equal 73, ary.binary_search(50) { |key, obj| obj <=> key }
    assert_equal 73, ary.binary_search(51) { |key, obj| obj <=> key }
    assert_equal nil, ary.binary_search(80) { |key, obj| obj <=> key }
    assert_equal 10, ary.binary_search(-3) { |key, obj| obj <=> key }
    assert_equal nil, [].binary_search(-3) { |key, obj| obj <=> key }
  end

  def test_asymmetric_comparison
    ary = [10, 20, 30, 45, 50, 73]
    assert_equal 10, ary.binary_search(10) { |key, obj| 2 * key <=> obj }
    assert_equal 20, ary.binary_search(11) { |key, obj| 2 * key <=> obj }
    assert_equal 50, ary.binary_search(35) { |key, obj| 2 * key <=> obj }
    assert_equal 73, ary.binary_search(50) { |key, obj| 2 * key <=> obj }
    assert_equal 73, ary.binary_search(51) { |key, obj| 2 * key <=> obj }
    assert_equal 73, ary.binary_search(80) { |key, obj| 2 * key <=> obj }
    assert_equal nil, ary.binary_search(-3) { |key, obj| 2 * key <=> obj }
    assert_equal nil, [].binary_search(-3) { |key, obj| 2 * key <=> obj }
  end

  def test_binary_search_return_option
    ary = [10, 20, 30, 45, 50, 73]
    assert_equal nil, ary.binary_search(10, :return => :object)
    assert_equal 10, ary.binary_search(11, :return => :object)
    assert_equal 2, ary.binary_search(35, :return => :index)
    assert_equal [45, 3], ary.binary_search(50, :return => :object_and_index)
    assert_equal [3, 45], ary.binary_search(51, :return => :index_and_object) { |key, obj| key - 4 <=> obj }
    assert_equal [5, 73], ary.binary_search(80, :return => :index_and_object)
    assert_equal nil, ary.binary_search(-3, :return => :object)
    assert_equal nil, [].binary_search(-3, :return => :object)
  end

  def test_search_direction
    ary = [10, 20, 30, 45, 50, 73]
    assert_equal 20, ary.binary_search(10, :find => :next_larger)
    assert_equal 20, ary.binary_search(11, :find => :next_larger)
    assert_equal 45, ary.binary_search(35, :find => :next_larger)
    assert_equal 73, ary.binary_search(50, :find => :next_larger)
    assert_equal 73, ary.binary_search(51, :find => :next_larger)
    assert_equal nil, ary.binary_search(80, :find => :next_larger)
    assert_equal 10, ary.binary_search(-3, :find => :next_larger)
    assert_equal nil, [].binary_search(-3, :find => :next_larger)
  end

end
