class SelectiveProfiler

  def self.global
    @@global ||= SelectiveProfiler.new
  end

  def initialize
    @stats = Hash.new { |hash, key| hash[key] = Hash.new { |hash, key| hash[key] = MethodStats.new } }
  end

  def register_start(klass, name)
    @stats[klass][name].register_start
  end

  def register_stop(klass, name)
    @stats[klass][name].register_stop
  end

  attr_reader :stats

  def stats_collected?
    @stats.keys.length > 0
  end

  def report
    result = pad("calls") + " " + pad("sec") + " " + pad("ms/call") + "  name"
    @stats.keys.sort { |a,b| a.to_s <=> b.to_s }.each do |klass|
      @stats[klass].keys.sort { |a,b| a.to_s <=> b.to_s }.each do |method|
        stats = @stats[klass][method]
        result += "\n#{pad(stats.total_calls)} #{output_float stats.total_time} #{output_float(stats.total_time*1000/stats.total_calls)}"
        result += "  #{klass}##{method}"
      end
    end
    result
  end

  private
  def pad(string, width = 8)
    " " * [0, width - string.to_s.length].max + string.to_s
  end

  def output_float(flt, width = 8)
    pad(((flt*100).truncate/100.0).to_s, width)
  end

end

class MethodStats

  attr_reader :total_calls, :total_time

  def initialize
    @total_time = 0
    @total_calls = 0
    @start_time = nil
  end

  def register_start
    @start_time = Time.now
  end

  def register_stop
    @total_time += Time.now - @start_time
    @total_calls += 1
    @start_time = nil
  end

end

class Module

  def instrument(name, profiler = SelectiveProfiler.global)
    uninstrumented_name = ("uninstrumented_" + name.to_s).to_sym
    alias_method uninstrumented_name, name
    define_method(name) do |*args|
      profiler.register_start(self.class, name)
      value = self.send uninstrumented_name, *args
      profiler.register_stop(self.class, name)
      value
    end
  end

end
