#!/usr/bin/perl -w

use strict;
use Carp;
use Data::Dumper;
use Getopt::Long;
use dbm;
#< documentation
use constant VERSION => q$Id: makedbm.pl,v 1.2 2006/01/18 18:39:30 gremio Exp $.$/;
use constant USAGE => <<"EOT"
usage: $0 textfile dbmfile
EOT
    ;
=pod

=head1 NAME

makedbm - make a single-layer mldbm hash out of a tab-delimited two-column text file

=head1 SYNOPSYS

makedbm.pl textfile dbmfile

=head1 DESCRIPTION

Given an input text with two columns: 
  a1 \t b1 \n
  a2 \t b2 \n
  a3 \t b3 \n
  ...
create a dbm of the form: 
  dbm = { a1 => b1, 
          a2 => b2,
          a3 => b3 }
replacing any duplicate keys. 

=head1 NOTES

=head1 SEE ALSO

=head1 BUGS

=cut
    ;
#> 

my $toeval;
GetOptions ("help" => sub {exec qq(perldoc $0);},
	    "Version" => sub {print VERSION;exit 0;},
	    "eval=s" => \$toeval,
	    ) or die USAGE;

my $dbmfile = pop @ARGV;
die USAGE unless @ARGV;
unlink $dbmfile if -e $dbmfile;
my $db = writedb($dbmfile);

my $esub = sub { return @_; };
if ($toeval) {
    $esub = eval "sub { $toeval }";
}
while (<>) {
    next if /^\#/ or !/\S/;
    if (/^([^\t]*)\t([^\n]*)\n$/) {
	$db->{$1} = &$esub($2);
    } else {
	warn "unexpected format [$_] at line $.\n";
    }
}

unless ($dbm::BERKELEY) {
    open(OUT, ">", $dbmfile) or die "cannot write [$dbmfile]: $!\n";
    print OUT Dumper($db);
    close OUT;
}

1;

