# -*- perl -*-

use Data::Dumper;
use constant keep_known_negs => 1;
my $keep_known_negs = keep_known_negs;
use Test::More tests=> (keep_known_negs) ? 99 : 19;

my $pwd=`pwd`; chomp($pwd);
my $n = $pwd;
$n =~ s|/Nuggeteer/.*?$|Nuggeteer|g;
$n = qq($n/src/nuggeteer.pl);
require $n;

my $jfilename = "/tmp/test-jfile$$";
#< setup jfile
open(JFILE, ">", $jfilename) or die "cannot write [$jfilename]: $!\n";
print JFILE << 'EndOfAssgFile'; 
Submission for question 1901 with items numbered:
1901 MITCSAIL03c 1 NYT19980818.0434 sentence one
1901 MITCSAIL03c 2 NYT19981202.0095 phrase two and so on
1901 MITCSAIL03c 3 NYT19981209.0243 word
1901 MITCSAIL03c 4 NYT19990517.0473 Among others
1901 MITCSAIL03c 5 NYT19990610.0211 in many dance works, especially if they had scores by Aaron Copland ("Billy the Kid," "Rodeo" and "Appalachian Spring")
1901 MITCSAIL03c 6 NYT19990708.0098 
1901 MITCSAIL03c 7 NYT19991115.0115 0
1901 MITCSAIL03c 8 NYT20000605.0143 1

Nuggets assigned to items for question 1901:
1901    MITCSAIL03c     1       11
1901    MITCSAIL03c     3       1
1901    MITCSAIL03c     5       2
1901    MITCSAIL03c     5       11



Submission for question 1901a with items numbered:
1901a MITCSAIL03c 1 NYT19980818.0434 sentence one

Nuggets assigned to items for question 1901a:


Submission for question 1905 with items numbered:
1905 MITCSAIL03c 1 NYT19981009.0448 a bc def ghij
1905 MITCSAIL03c 2 NYT19981020.0376 abcd efg hi j
1905 MITCSAIL03c 3 NYT20000612.0198 among others

Nuggets assigned to items for question 1905:
1905    MITCSAIL03c     1       1
1905    MITCSAIL03c     3       1
EndOfAssgFile
;
close JFILE;
#>
#< setup
my %opt = ( qidre => q([a-zA-Z\d\.]+),
	    quiet => 1 );
my %cmpt = ( word_selection_sub => \&Idf::words );
my %jdb = (_settings => { stopwords => "keep", 
			  jfiles => [ $jfilename, $jfilename ],
			  max_judgement_rank => 0 },
	   _questions => {"1901" => "question 1901",
			  "1901a" => "another version",
			  1905 => "a later question"},
	   _nuggets => {1901 => {1=>"one", 2=>"two",3=>"three",4=>"four",
				 5=>"five",6=>"six",7=>"seven",8=>"eight",
				 9=>"nine",10=>"ten",11=>"eleven"},
			"1901a" => {1=>"egy",2=>"ketto"},
			1905 => {1=>"une"},
	   },
	   _vitoks => {1901 => {1=>1,2=>1,3=>1,4=>1,5=>1,6=>1,7=>1,8=>1,
				9=>1,10=>0,11=>0},
		       "1901a" => {1=>1,2=>0.5},
		       1905 => {1=>0,2=>1,3=>1},
	   },
	   _qids => [qw(1901 1901a 1905)],
    ); # I don't need learn_strings for reading judgements
#>
my ($nresp, $nassg) = Nuggeteer::read_judgements(\%jdb, \%opt, \%cmpt);

#< test_result (2)
sub test_result {
    is ($nresp, 2 * 12, "number of responses in test file");
    is ($nassg, 2 *  6, "number of assignments in test file");
}
#>
#< test_known_neg (64+3)
sub test_known_neg { # {qid => { nid => { response => 1 }}
    if ($keep_known_negs) {
    for my $nid (3, 4, 5, 6, 7, 8, 9, 10) {
	is ($jdb{_known_neg}{1901}{$nid}{"sentence one"}, 1, "_kn 1901;$nid r1");
	is ($jdb{_known_neg}{1901}{$nid}{"phrase two and so on"}, 1, "_kn 1901;$nid r2");
	is ($jdb{_known_neg}{1901}{$nid}{"word"}, 1, "_kn 1901;$nid r3");
	is ($jdb{_known_neg}{1901}{$nid}{"among others"}, 1, "_kn 1901;$nid r4");
	is ($jdb{_known_neg}{1901}{$nid}{'in many dance works, especially if they had scores by aaron copland ("billy the kid," "rodeo" and "appalachian spring")'}, 1, "_kn 1901;$nid r5");
	is ($jdb{_known_neg}{1901}{$nid}{""}, 1, "_kn 1901;$nid r=empty string");
	is ($jdb{_known_neg}{1901}{$nid}{"0"}, 1, "_kn 1901;$nid r=zero");
	is ($jdb{_known_neg}{1901}{$nid}{"1"}, 1, "_kn 1901;$nid r=1");
    }
    }
    for my $nid (1, 2, 11) {
	ok(not(exists($jdb{_known_neg}{1901}{$nid})), "nid $nid is not known negative");
    }
}
#>
#< test_known_pos (10)
sub test_known_pos {
    is($jdb{_known_pos}{1901}{11}{'sentence one'}, 1, "1901;r1;11");
    is($jdb{_known_pos}{1901}{11}{'in many dance works, especially if they had scores by aaron copland ("billy the kid," "rodeo" and "appalachian spring")'}, 1, "1901;r5;11");
    is($jdb{_known_pos}{1901}{1}{'word'}, 1, "1901;r3;1");
    is($jdb{_known_pos}{1901}{2}{'in many dance works, especially if they had scores by aaron copland ("billy the kid," "rodeo" and "appalachian spring")'}, 1, "1901;r5;2");
    is($jdb{_known_pos}{1905}{1}{'a bc def ghij'}, 1, "1905;r1;1");
    is($jdb{_known_pos}{1905}{1}{'among others'}, 1, "1905;r3;1");

    is(scalar(keys %{$jdb{_known_pos}{1901}{1}}), 1, "size _kp 1901;1");
    is(scalar(keys %{$jdb{_known_pos}{1901}{2}}), 1, "size _kp 1901;2");
    is(scalar(keys %{$jdb{_known_pos}{1901}{11}}), 2, "size _kp 1901;11");
    is(scalar(keys %{$jdb{_known_pos}{1905}{1}}), 2, "size _kp 1905;1");
}
#>
#< test_t_response (3+12+4)
sub test_t_response { # test '\t response' entries

    is($jdb{"\tword"}{1901}{1},     1, "1901;3;1 assigned true");
    is($jdb{"\tword"}{1901}{2}, undef, "1901;3;2 not assigned");
    is($jdb{"\tword"}{1901}{3},     0, "1901;3;3 assigned false") if $keep_known_negs;
    my %total_pos = ();
    foreach my $entry (keys %jdb) {
	if ($entry =~ /^\t/) { # this is a response containing:
	    # { qid => { nid => true if known positive } }
	    foreach my $qid (keys %{$jdb{$entry}}) {
		my $known = scalar(keys %{$jdb{$entry}{$qid}});
		my $known_neg = scalar(keys %{$jdb{_known_neg}{$qid}});
		ok($known >= $known_neg, "$qid$entry has at least known_neg nuggets") if $keep_known_negs;
		$total_pos{$qid} += ($known - $known_neg);
	    }
	}
    }

    # _known_pos will not have 1901a but total_pos will, because 1901a
    # has no known positives, but it does appear in \tresponse
    # entries.
    foreach my $qid (keys %{$jdb{_known_pos}}, keys %total_pos) {
	my $known_pos = 0;
	foreach my $nid (keys %{$jdb{_known_pos}{$qid}}) {
	    $known_pos += scalar keys %{$jdb{_known_pos}{$qid}{$nid}};
	}
	is ($known_pos, $total_pos{$qid}, "known = total pos for $qid") 
	    if $keep_known_negs;
    }

}

#>
#< test_max_judg(2)
sub test_max_judg {
    my %ljdb = %jdb;
    $ljdb{_settings}{max_judgement_rank} = 1;
    my ($nresp, $nassg) = Nuggeteer::read_judgements(\%ljdb, \%opt, \%cmpt);
    is ($nresp, 2 * 3, "number of responses in test file, reading only 1");
    is ($nassg, 2 * 2, "number of assignments in test file, reading only 1");
}
#>

&test_result();
&test_known_neg();
&test_known_pos();
&test_t_response();
&test_max_judg();
#warn Dumper (\%jdb);
unlink $jfilename;
