# -*- perl -*-

# Common helpers used in the test suite live here.  The test suite
# require's this file.

#< get_rid_of(files)
sub get_rid_of {
    unlink @_;
}
#>
#< program_runs(program_name)
sub program_runs {
    system("$_[0] --version > /dev/null") == 0;
}
#>
#< ensure_dir_exists(files)
sub ensure_dir_exists {
    use File::Basename;
    use File::Path;
    foreach my $filename (@_) {
	my $dirname = dirname($filename);
	mkpath $dirname;
    }
}
#>
#< runok(cmd, name)
# Performs one test
sub runok {
    my ($cmd, $outfile, $errfile, $name) = @_;
    get_rid_of($outfile, $errfile);
    ensure_dir_exists($outfile, $errfile);
    $cmd .= qq( > $outfile 2> $errfile);
    if(system($cmd)) {
	print STDERR "trouble running [$cmd]: signal: " . ($? & 127) . " exit status: " . ($? >> 8) . " $@\n";
	print STDERR `cat $errfile`;
	fail("running $name: $@\n");
	exit 1;
    } else {
	pass("running $name");
    }
}
#>
#< check_genstats(file, n, f, ci, name)
# Performs two tests, exits the process if any fail.
# The exit is so done to avoid diffing the output if it is already
# known that the answers diverge.
sub check_genstats {
    my ($file, $n, $f, $ci, $name) = @_;
    my $g1=`genstats.pl -b $file 8`;
    ok($g1, "no output from exact") or exit 1;
    is($g1, "n= $n	col1:	$f +- $ci\t\n", "f-measures for $name")
	or exit 1;
}
#>
#< same_output(file1, file2, name)
# Performs one test
sub same_output {
    my ($file1, $file2, $name) = @_;
    my $diff1=`diff $file1 $file2 2>&1`;
    $diff1=~ s/^. . (nuggeteer|user|host|global threshold|answerkey|judgement files|testfile) = .*//mg; # trivial settings
    $diff1=~ s/^. . starting at: .*//mg; # script start time
    $diff1=~ s/^. .EXTERN_COMMAND: .*//mg; # external command can have temp file names
    $diff1=~ s/^[\s-]*$//mg; #diff output
    $diff1=~ s/^[\d,]+[adc][\d,]+$//mg; #diff output
    $diff1=~ s/\n+/\n/gs; # compress cleared lines above
    is($diff1, "\n", "diff $name");
}
#>

1;
